
package stamp.peripheral.display.lcd.CrystalFontz;
import stamp.core.*;
import stamp.peripheral.sensor.GPS.DisplayDriver.*;

public class CrystalFontz
{

  public final static int NUMBER_OF_COLS = 20;
  public final static int NUMBER_OF_ROWS = 4;
  private int[] m_LCDLine1;
  private int[] m_LCDLine2;
  private int[] m_LCDLine3;
  private int[] m_LCDLine4;
  private Uart m_uart;

  public CrystalFontz(Uart uart)
  {
    m_uart = uart;
    m_LCDLine1 = new int[NUMBER_OF_COLS];
    m_LCDLine2 = new int[NUMBER_OF_COLS];
    m_LCDLine3 = new int[NUMBER_OF_COLS];
    m_LCDLine4 = new int[NUMBER_OF_COLS];

    // Set the correct contrast
    m_uart.sendByte((byte)15);
    m_uart.sendByte((byte)70);

  }
  public void Clear()
  {
    // Clear the screen too
    ClearDisplay();
  }
  public void WriteCharacter(int nRow, int nColumn, byte ch)
  {
    // Boundary check
    if ((nRow>=4)||(nColumn)>19) return;
    int[] row = GetRow(nRow);
    row[nColumn] = (0x0100 | ch); // We have a new row here

    // Let this function display the changed data
    DisplayData(nRow, false);
  }
  public void DisplayText(int nRow, int nStartCol, String buffer)
  {
    // If we are to start from the last location
    //if (nRow==-1) nRow=nLastRow;
    //if (nStartCol==-1) nStartCol=(m_LastColumn+1);

    // Boundary check
    if (nRow>=4) return;

    //int nBufferLength = buffer.length();

    int[] row = GetRow(nRow);
    for (int n=0; n<buffer.length(); n++)
    {
      // We can only have 20 characters on screen, so sending more characters than that
      if ((nStartCol+n)>19)
        break;

      char chNew = buffer.charAt(n);
      char chOld = (char)(row[nStartCol+n] & 0x00FF);
      if (chOld!=chNew)
        row[nStartCol+n] = (0x0100 | (byte)chNew); // We have a new row here
    }
    // Let this function display the changed data
    DisplayData(nRow, false);
  }
  // Function moves the cursor to a position
  // When sending data to the display, sending a character automatically moves the cursor to the next position
  public void Goto(int nRow, int nCol)
  {
    m_uart.sendByte(17);
    m_uart.sendByte((byte)nCol);
    m_uart.sendByte((byte)nRow);
  }
  private void DisplayData(int nRow, boolean bRefreshAnyway)
  {
    // Display only whats beeing changed
    switch (nRow)
    {
      case 0: DisplayData(0, m_LCDLine1, bRefreshAnyway); break;
      case 1: DisplayData(1, m_LCDLine2, bRefreshAnyway); break;
      case 2: DisplayData(2, m_LCDLine3, bRefreshAnyway); break;
      default:
       DisplayData(3, m_LCDLine4, bRefreshAnyway); break;
    }
  }
  private void DisplayData(int nRow, int[] row, boolean bRefreshAnyway)
  {
    for (int nColumn=0; nColumn<NUMBER_OF_COLS; nColumn++)
      if (((row[nColumn] & 0x0100)>0) || bRefreshAnyway)
      {
        Goto(nRow, nColumn);
        int nChar = (int)((byte)row[nColumn]&0x00FF);
        switch( nChar )
        {
          case DisplayDriver.SPECIAL_CHARACTER_DEGREE:
          case DisplayDriver.SPECIAL_CHARACTER_ARROW_RIGHT:
          {
            m_uart.sendByte(30);
            m_uart.sendByte(1);
            if ( DisplayDriver.SPECIAL_CHARACTER_DEGREE == nChar )
              m_uart.sendByte(128);
            else
              m_uart.sendByte(16);
          }
          break;
          default:
            m_uart.sendByte((byte)(row[nColumn] & 0x00FF)); break;// Send the byte to the display


        }
        // Mark this row as old (so we don't show it again)
        row[nColumn] &= 0x00FF;
      }
  }
  public int[] GetRow(int nRow)
  {
    switch (nRow)
    {
      case 0: return (m_LCDLine1);
      case 1: return (m_LCDLine2);
      case 2: return (m_LCDLine3);
      default:
       return (m_LCDLine4);
    }
  }
  public void ClearRange(int nStartRow, int nEndRow, int nStartColumn, int nEndColumn)
  {
    for (int nRow=nStartRow; nRow<nEndRow; nRow++)
    {
      int[] row = GetRow(nRow); // Clear all the rows
      for (int n=nStartColumn; n<nEndColumn; n++)
      {
        m_uart.sendByte(17);
        m_uart.sendByte((byte)n);
        m_uart.sendByte((byte)nRow);

        m_uart.sendByte((byte)' ');
        row[n]=0x0000;
      }
    }

    m_uart.sendByte(1);
  }
  public void ClearDisplay()
  {
    ClearRange(0, NUMBER_OF_ROWS, 0, NUMBER_OF_COLS);
  }
  private void ClearRow(int[] Row)
  {
    for (int nColumn=0; nColumn<NUMBER_OF_COLS; nColumn++)
      Row[nColumn]=0x0000;
  }
  public void ShowCursor(boolean bShow)
  {
    m_uart.sendByte( bShow ? 6 : 4);
  }
  public void ScrollOff(boolean bScrollOff)
  {
    m_uart.sendByte( bScrollOff ? 20 : 19);
  }
}