package stamp.peripheral.sensor.GPS.DisplayDriver;

import stamp.core.*;
import stamp.peripheral.display.lcd.CrystalFontz.*;

/**
This class is handling the output...if the debug mode, write to System.out, in other case write to "20x4 CrystalFontz"
*/
public class DisplayDriver
{
  public static final int DISPLAY_1 = 0x0001;
  public static final int DISPLAY_2 = 0x0002;

  private CrystalFontz m_CrystalFontzLCD1;
  private CrystalFontz m_CrystalFontzLCD2;

  public static final int SPECIAL_CHARACTER_DEGREE        = 221;    //
  public static final int SPECIAL_CHARACTER_HEART_BEAT    = 147;    // A heart
  public static final int SPECIAL_CHARACTER_GPS_COVERAGE  = 149;    // This character looks like an antenna
  public static final int SPECIAL_CHARACTER_ARROW_RIGHT   = 16;     // Looks like an arrow pointing to the right


  public DisplayDriver(CrystalFontz display1, CrystalFontz display2)
  {
    m_CrystalFontzLCD1 = display1;
    m_CrystalFontzLCD2 = display2;
  }

  public void ClearScreenRange(int nDisplay, int nStartRow, int nEndRow, int nStartColumn, int nEndColumn)
  {
    if ((nDisplay & DISPLAY_1)>0)
      m_CrystalFontzLCD1.ClearRange( nStartRow, nEndRow, nStartColumn, nEndColumn );

    if ((nDisplay & DISPLAY_2)>0)
      m_CrystalFontzLCD2.ClearRange( nStartRow, nEndRow, nStartColumn, nEndColumn );
  }
  public void ClearScreen(int nDisplay)
  {
    if ((nDisplay & DISPLAY_1)>0)
      m_CrystalFontzLCD1.Clear();

    if ((nDisplay & DISPLAY_2)>0)
      m_CrystalFontzLCD2.Clear();

  }
  public void WriteMessage(int nDisplay, int nRow, int nColumn, String str)
  {
    if ((nDisplay & DISPLAY_1)>0)
      m_CrystalFontzLCD1.DisplayText(nRow, nColumn, str);

    if ((nDisplay & DISPLAY_2)>0)
      m_CrystalFontzLCD2.DisplayText(nRow, nColumn, str);
  }
  public void WriteCharacter(int nDisplay, int nRow, int nColumn, byte ch)
  {
    if ((nDisplay & DISPLAY_1)>0)
      m_CrystalFontzLCD1.WriteCharacter(nRow, nColumn, ch);

    if ((nDisplay & DISPLAY_2)>0)
      m_CrystalFontzLCD2.WriteCharacter(nRow, nColumn, ch);
  }
  public void ScrollOff(int nDisplay, boolean bScrollOff)
  {
      if ((nDisplay & DISPLAY_1)>0)
        m_CrystalFontzLCD1.ScrollOff(bScrollOff);
      if ((nDisplay & DISPLAY_2)>0)
        m_CrystalFontzLCD2.ScrollOff(bScrollOff);
  }
  public void ShowCursor(int nDisplay, boolean bShow)
  {
      if ((nDisplay & DISPLAY_1)>0)
        m_CrystalFontzLCD1.ShowCursor(bShow);
      if ((nDisplay & DISPLAY_2)>0)
        m_CrystalFontzLCD2.ShowCursor(bShow);
  }

  public void Goto(int nDisplay, int nRow, int nCol)
  {
      if ((nDisplay & DISPLAY_1)>0)
        m_CrystalFontzLCD1.Goto(nRow, nCol);
      if ((nDisplay & DISPLAY_2)>0)
        m_CrystalFontzLCD2.Goto(nRow, nCol);
  }

  public int[] GetRow(int nDisplay, int nRow)
  {
      if ((nDisplay & DISPLAY_1)>0)
        return m_CrystalFontzLCD1.GetRow(nRow);


    // Assume the second LCD
    return m_CrystalFontzLCD2.GetRow(nRow);
  }
}