package stamp.peripheral.sensor.GPS.Include;           // Contains the linked list class and other stuff

import stamp.core.*;
import com.micromegacorp.math.hardware.*;

public class BacklightHandler
{
  byte      m_CurrentReading;                       // The current reading... (0-10)
  boolean   m_bAllReadings;                         // Variable is true when all the 10 readings have been
  int[]     m_nAccumulatedValues = new int[5];      // All the 10 previous readings
  boolean   m_bBackLightState = true;               // The light is ON as default

  public BacklightHandler()
  {
    m_CurrentReading = 0;
  }

  public void Idle(Float32 float32)
  {
    CPU.writePin(CPU.pin0, true);
    CPU.delay(10);

    int nValue = CPU.rcTime( 10000, CPU.pin0, false );
    m_nAccumulatedValues[GetNextReadingIndex()] = (nValue == -1) ? 30000 : nValue;
    Process(float32);
  }

  private byte GetNextReadingIndex()
  {
    if (m_CurrentReading >= 5)
    {
      m_CurrentReading = 0;
      m_bAllReadings = true;
      return m_CurrentReading;
    }

    return (m_CurrentReading++);
  }
  private void Process(Float32 float32)
  {
    // Nothing to do here
    if ( !m_bAllReadings ) return;
    float32.set(0);

    int nValue = 0;
    for ( int i = 0; i < 5; i++ )
      float32.add( m_nAccumulatedValues[i] );

    float32.divide( 5 );
    boolean bLightsOn = ( float32.compare(2000) >=0 );
    if ( m_bBackLightState != bLightsOn )
    {
      for ( int i = 0; i < 5; i++ )
        m_nAccumulatedValues[i] = bLightsOn ? 30000 : 1;
    }

    m_bBackLightState = bLightsOn;
    CPU.writePin(CPU.pin9, bLightsOn);
    CPU.writePin(CPU.pin1, bLightsOn);
  }
}