
package stamp.peripheral.sensor.GPS.Include;
import stamp.peripheral.sensor.GPS.Include.*;
import stamp.peripheral.sensor.GPS.NMEA.*;

import stamp.core.*;

/**
  This class contains a linked list which is holding the GPS messages
  received. It's a "double liked list" but does not contain any z-nodes,
  looping trough the list will be endeless.

*/

public class GPSMessageList
{
  // The "item" contains the pointer to the first position in list
  private GPSMessageItem HeadItem;
  private GPSMessageItem LastUsedItem;

  public GPSMessageList()
  {
    HeadItem = null;
    LastUsedItem = null;
  }

  /**
    This function will move forward to next item in the "circle"
  */
  public GPSMessageItem MoveForwardToNextMessage()
  {
    // Return the last used item...
    LastUsedItem = GetNextItem( LastUsedItem );

    // Now, the "LastUsedItem" will point to next item in the list
    return ( LastUsedItem );
  }

  /**
    This function will move backward to previous item in the "circle"
  */
  public GPSMessageItem MoveBackwardToPreviousMessage()
  {
    // Move to forward item in the list
    // This item will be saved in this class
    LastUsedItem = GetPrevItem( LastUsedItem );

    return ( LastUsedItem );
  }
  public GPSMessageItem GetCurrentMessage(){ return (LastUsedItem);}

  // Function creates a message list
  public void CreateMessageList( int nType, int nNumberOfItems )
  {

   GPSMessageItem lastItem = HeadItem;

    for ( int i = 0; i < nNumberOfItems; i++ )
    {
      // Create a new pointer based on the type
      GPSMessageItem newItem = CreateNewItem( nType );
      if ( HeadItem == null )
      {
        HeadItem = newItem;
        lastItem = newItem;
      }

      // Fix the pointers
      lastItem.SetNextItem(newItem);
      newItem.SetPrevItem(lastItem);
      lastItem = newItem;
    }

    // After creating all the objects, the first item in the list - ItemFirst - should be linked with the last item in the list
    lastItem.SetNextItem(HeadItem);
    HeadItem.SetPrevItem(lastItem);
  }

  private GPSMessageItem GetNextItem( GPSMessageItem gpsMessage )
  {
    if (gpsMessage == null)
      return HeadItem;

    return (gpsMessage.GetNextItem());
  }
  private GPSMessageItem GetPrevItem( GPSMessageItem gpsMessage )
  {
    if (gpsMessage == null)
      return HeadItem;

    return (gpsMessage.GetPrevItem());
  }


  private GPSMessageItem CreateNewItem( int nType )
  {
    switch (nType)
    {
      //case NMEADefinitions.GGA_MESSAGE: return ((GPSMessageItem)(new GGA(m_globalVars.m_timeParser)));
      case NMEADefinitions.VTG_MESSAGE: return ((GPSMessageItem)(new VTG()));
      default:
        return (GPSMessageItem)null;
    }
  }
}