
package stamp.peripheral.sensor.GPS.Keyboard;
import stamp.core.*;
import stamp.peripheral.sensor.GPS.DisplayDriver.*;     // Contains the LCD driver for the CrystalFontz

public class MEMKey
{
  Uart m_rxMemKey;
  public static final int KEYBOARD_MODE_ALFA        = 0x0001;     // The return value are characters - (A->Y)
  public static final int KEYBOARD_MODE_NUMERIC     = 0x0002;     // Only numbers (0-9)
  public static final int KEYBOARD_MODE_EXTRA       = 0x0004;     // extra keys: A, B, C, D
  public static final int KEYBOARD_MODE_SPECIAL     = 0x0008;     // special keys: *, #
  public static final int KEYBOARD_MODE_INTERACTIVE = 0x0100;     // if the result should be shown in the LCD in the same time
  public static final int KEYBOARD_MODE_ENTER_TEXT  = 0x0200;     // if the user should enter text

  public static final byte KEYBOARD_NONE_BYTE       = 16;
  public static final byte KEYBOARD_BYTE_A          = 3;          // 'A'
  public static final byte KEYBOARD_BYTE_B          = 7;          // 'B'
  public static final byte KEYBOARD_BYTE_C          = 11;         // 'C'
  public static final byte KEYBOARD_BYTE_STAR       = 12;         // '*'
  public static final byte KEYBOARD_BYTE_D          = 15;         // 'D'


  private MEMKeyInput m_KeyboardInput;

  private static char[] m_charsA = {'a', 'b', 'c', 'A', 'B', 'C', '1', '2'};
  private static char[] m_charsD = {'d', 'e', 'f', 'D', 'E', 'F', '3'};
  private static char[] m_charsG = {'g', 'h', 'i', 'G', 'H', 'i', '4'};
  private static char[] m_charsJ = {'j', 'k', 'l', 'J', 'K', 'L', '5'};
  private static char[] m_charsM = {'m', 'n', 'o', 'M', 'N', 'O', '6'};
  private static char[] m_charsP = {'p', 'r', 's', 'P', 'R', 'S', '7'};
  private static char[] m_charsT = {'t', 'u', 'v', 'T', 'U', 'V', '8'};
  private static char[] m_charsW = {'w', 'x', 'y', 'W', 'X', 'Y', '9'};
  private static char[] m_charsSpecial = {'-', '+', '*', '/', '<', '>', ':', ';', '#', '%', '=', '"', '!', '&', '(', ')'};


  public MEMKey()
  {
    m_rxMemKey = new Uart( Uart.dirReceive,
                           CPU.pin15,
                           Uart.dontInvert,
                           Uart.speed2400,
                           Uart.stop1  );


    CPU.writePin( CPU.pin11, true ); // Let the noise settle
    CPU.delay( 2000 );

    // Prepare to send data
    m_rxMemKey.setDirection( Uart.dirTransmit );
    m_rxMemKey.sendByte( 0x11 ); // Send the reset command

    // Restore the transmission to receive and start reading the data we get
    m_rxMemKey.setDirection( Uart.dirReceive );

    m_KeyboardInput = new MEMKeyInput();
  }


  public MEMKeyInput GetCurrentChar(int nMode)
  {
    m_KeyboardInput.Clear();
    if (m_rxMemKey.byteAvailable())
      GetChar(nMode, KEYBOARD_NONE_BYTE, KEYBOARD_NONE_BYTE, m_KeyboardInput, false );

    ClearMEMKeyBuffer();

    return (m_KeyboardInput);
  }
  public void ClearMEMKeyBuffer()
  {
    while ( m_rxMemKey.byteAvailable() )
      m_rxMemKey.receiveByte();
  }
  private boolean GetChar(int nMode, byte nEscapeByte1, byte nEscapeByte2, MEMKeyInput keyInfo, boolean bRetry)
  {
    // Wait until there is a character available
    while ( !m_rxMemKey.byteAvailable() ) { CPU.delay( 250 ); }
    byte b = (byte)m_rxMemKey.receiveByte();
    keyInfo.m_Key = b;
    if ( ( b == nEscapeByte1 ) || ( b == nEscapeByte2 ) )
      return true;

    keyInfo.m_byteSeries = null;

    switch ( nMode )
    {
      case KEYBOARD_MODE_ALFA:  // The return value are characters - (A->Y)
      {
        switch (b)
        {
          case 1: keyInfo.m_Alfa = (char)'A'; keyInfo.m_byteSeries = m_charsA; return true;
          case 2: keyInfo.m_Alfa = (char)'D'; keyInfo.m_byteSeries = m_charsD; return true;
          case 4: keyInfo.m_Alfa = (char)'G'; keyInfo.m_byteSeries = m_charsG; return true;
          case 5: keyInfo.m_Alfa = (char)'J'; keyInfo.m_byteSeries = m_charsJ; return true;
          case 6: keyInfo.m_Alfa = (char)'M'; keyInfo.m_byteSeries = m_charsM; return true;
          case 8: keyInfo.m_Alfa = (char)'P'; keyInfo.m_byteSeries = m_charsP; return true;
          case 9: keyInfo.m_Alfa = (char)'T'; keyInfo.m_byteSeries = m_charsT; return true;
          case 10: keyInfo.m_Alfa = (char)'W';  keyInfo.m_byteSeries = m_charsW; return true;
          case 13: keyInfo.m_Alfa = (char)'-';  keyInfo.m_byteSeries = m_charsSpecial; return true;

          default:
            keyInfo.m_Alfa = 0;

        }
      }
      break;

      case KEYBOARD_MODE_NUMERIC:       // Only numbers (0-9)
      {
          switch (b)
          {
            case 0: keyInfo.m_Numeric = 1; keyInfo.m_Alfa = '1'; return true;
            case 1: keyInfo.m_Numeric = 2; keyInfo.m_Alfa = '2'; return true;
            case 2: keyInfo.m_Numeric = 3; keyInfo.m_Alfa = '3'; return true;
            case 4: keyInfo.m_Numeric = 4; keyInfo.m_Alfa = '4'; return true;
            case 5: keyInfo.m_Numeric = 5; keyInfo.m_Alfa = '5'; return true;
            case 6: keyInfo.m_Numeric = 6; keyInfo.m_Alfa = '6'; return true;
            case 8: keyInfo.m_Numeric = 7; keyInfo.m_Alfa = '7'; return true;
            case 9: keyInfo.m_Numeric = 8; keyInfo.m_Alfa = '8'; return true;
            case 10: keyInfo.m_Numeric = 9; keyInfo.m_Alfa = '9';return true;
            case 13: keyInfo.m_Numeric = 0; keyInfo.m_Alfa = '0'; return true;
            default:
              if ( bRetry )
                return (false);
          }
      }
      break;
      case KEYBOARD_MODE_SPECIAL:     // special keys: *, #
      {
          switch (b)
          {
            case 12: keyInfo.m_Special = (char)'*'; return true;
            case 14: keyInfo.m_Special = (char)'#'; return true;
            default:
            if ( bRetry )
                return (false);
          }
      }
      break;

      case KEYBOARD_MODE_EXTRA:     // extra keys: A, B, C, D
      {
          switch (b)
          {
            case 3:  keyInfo.m_Extra = (char)'A'; return true;
            case 7:  keyInfo.m_Extra = (char)'B'; return true;
            case 11: keyInfo.m_Extra = (char)'C'; return true;
            case 15: keyInfo.m_Extra = (char)'D'; return true;
            default:
            if ( bRetry )
              return (false);
          }
      }
      break;

      default: break;
    }

    return (true);
  }
  //
  public MEMKeyInput GetChar(boolean bClearFirst,
                             int nWhat,
                             DisplayDriver dispDriver,
                             int nDisplay,
                             int nRow,
                             int nColumn,
                             StringBuffer buffer,
                             int nLimit,
                             byte nEscapeByte1,
                             byte nEscapeByte2,
                             byte nByteNext,
                             byte nPrevious,
                             byte nClear )
  {
    buffer.clear();
    m_KeyboardInput.Clear();
    byte key = 0x0;

    // Usually, when we are expecting a character due to user interaction in a menu
    // clear the data first
    if (bClearFirst)
      while (m_rxMemKey.byteAvailable()){m_rxMemKey.receiveByte();}

    // Get the next character or iterate if the same key
    if ((nWhat & KEYBOARD_MODE_ENTER_TEXT) > 0)
    {
      dispDriver.Goto(nDisplay, nRow, nColumn);
      dispDriver.ShowCursor(nDisplay, true);
      GetChar(KEYBOARD_MODE_ALFA, nEscapeByte1, nEscapeByte2, m_KeyboardInput, true);
      byte bLastKey = KEYBOARD_NONE_BYTE;
      char chLastCharacter = 0;

      int nPlace = 0;
      int nCurrentColumn = nColumn;
      while( (nCurrentColumn - nColumn ) < nLimit )
      {
        // If the same key is pushed...then iterate...
        if (bLastKey == KEYBOARD_NONE_BYTE )
          bLastKey = m_KeyboardInput.m_Key;

        if ((m_KeyboardInput.m_Key == nByteNext) || (m_KeyboardInput.m_Key == nPrevious))
        {

          if (m_KeyboardInput.m_Key == nByteNext) nCurrentColumn++;
          else if (m_KeyboardInput.m_Key == nPrevious)
            if (nCurrentColumn>nColumn) nCurrentColumn--;


          dispDriver.Goto(nDisplay, nRow, nCurrentColumn);
          nPlace = 0;
          // Get the next character
          GetChar(KEYBOARD_MODE_ALFA, nEscapeByte1, nEscapeByte2, m_KeyboardInput, true);
          bLastKey = m_KeyboardInput.m_Key;
          continue;
        }
        else if (m_KeyboardInput.m_Key == nClear)
        {
          dispDriver.Goto(nDisplay, nRow, nCurrentColumn);
          dispDriver.WriteCharacter(nDisplay, nRow, nCurrentColumn, (byte)' ');
          dispDriver.Goto(nDisplay, nRow, nCurrentColumn);
          nPlace = 0;

          GetChar(KEYBOARD_MODE_ALFA, nEscapeByte1, nEscapeByte2, m_KeyboardInput, true);
          bLastKey = m_KeyboardInput.m_Key;
          continue;
        }

        if (bLastKey != m_KeyboardInput.m_Key)
        {
          nPlace = 0;
          if (nCurrentColumn<20) nCurrentColumn++;

          bLastKey = m_KeyboardInput.m_Key;
        }

        if ((m_KeyboardInput.m_Key == nEscapeByte1) || (m_KeyboardInput.m_Key == nEscapeByte2))
          break;

        // The same key is pressed...so iterate throught the character series
        char[] chSeries = m_KeyboardInput.m_byteSeries;
        if ( chSeries != null )
        {
          if ( nPlace >= chSeries.length )
            nPlace = 0;

          chLastCharacter = chSeries[nPlace];
        }
        dispDriver.WriteCharacter(nDisplay, nRow, nCurrentColumn, (byte)chLastCharacter);
        nPlace++;

        dispDriver.Goto(nDisplay, nRow, nCurrentColumn);

        // Get the next character
        GetChar(KEYBOARD_MODE_ALFA, nEscapeByte1, nEscapeByte2, m_KeyboardInput, true);
      }

      // Get the information from the LCD
      int[] Row = dispDriver.GetRow(nDisplay, nRow);
      for (int nCol = nColumn; nCol < nLimit+nColumn; nCol++)
        buffer.append( (char)(Row[nCol] & 0x00FF) );

      dispDriver.ShowCursor(nDisplay, false);
    }
    else
    {
      // KEYBOARD_MODE_NUMERIC
      // KEYBOARD_MODE_EXTRA
      // KEYBOARD_MODE_SPECIAL
      while (!GetChar(nWhat, nEscapeByte1, nEscapeByte2, m_KeyboardInput, true))
      {
        // Get next character
      }
    }


    return (m_KeyboardInput);
  }
}